# noqa
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework.config import ConfigIO
from plumbum.cmd import docker

class Command(DodoCommand):  # noqa
    help = ""
    decorators = []

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('container_type')
        parser.add_argument('name')

    def handle_imp(self, container_type, name, **kwargs):  # noqa
        existing = docker("ps", "-a", "--quiet", "--filter", "name=" + name)
        if not existing:
            args = [
                "docker",
                "create",
                "--name", name,
            ]

            for path in self.get_config(
                "/DOCKER/container_types/%s/dirs" % container_type
            ):
                args.extend(["-v", path])

            args.append(self.get_config(
                "/DOCKER/container_types/%s/image" % container_type
            ))
            self.runcmd(args)

        config = ConfigIO().load(load_layers=False)
        config['DOCKER']['containers'][container_type] = name
        ConfigIO().save(config)



"""This command opens a bash shell in the docker container."""
from . import DodoCommand


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('command', nargs='?')
        parser.add_argument('--name')

    def handle_imp(self, command, name, **kwargs):  # noqa
        options = dict(
            self.get_config('/DOCKER/options/%s' % name, {})
        )
        options['name'] = name
        self.config['DOCKER']['options'].setdefault('docker', options)

        self.runcmd(
            ["/bin/bash"] + (["-c", command] if command else []),
            cwd=self.get_config("/DOCKER/default_cwd", None))



# noqa
from dodo_commands.system_commands import DodoCommand
from plumbum.cmd import docker
from six.moves import input as raw_input


class Command(DodoCommand):  # noqa
    help = ""

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('name', nargs='?')

    def handle_imp(self, name, **kwargs):  # noqa
        if not name:
            containers = [
                x for x in
                docker("ps", "--format", "{{.Names}}").split('\n')
                if x
            ]
            print("0 - exit")
            for idx, container in enumerate(containers):
                print("%d - %s" % (idx + 1, container))

            print("\nSelect a container: ")
            choice = int(raw_input()) - 1

            if choice == -1:
                return
            name = containers[choice]

        self.runcmd(
            [
                'docker',
                'exec',
                '-i',
                '-t',
                name,
                '/bin/bash',
            ],
        )



"""Build a docker image."""

import argparse
import os

from . import DodoCommand
from dodo_commands.framework import CommandError
from dodo_commands.framework.util import remove_trailing_dashes


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'name',
            choices=self.get_config('/DOCKER/images', {}).keys(),
            help='Look the docker image by this name in /DOCKER/images'
        )
        parser.add_argument(
            'build_args',
            nargs=argparse.REMAINDER
        )

    def _copy_extra_dirs(self, local_dir, extra_dirs):
        for extra_dir_name, extra_dir_path in extra_dirs.items():
            local_path = os.path.join(local_dir, extra_dir_name)
            if os.path.exists(local_path):
                raise CommandError(
                    "Cannot copy to existing path: " + local_path
                )
            if not os.path.exists(extra_dir_path):
                raise CommandError(
                    "Cannot copy from non-existing path: " + extra_dir_path
                )
            self.runcmd(["cp", "-rf", extra_dir_path, local_path])

    def _remove_extra_dirs(self, local_dir, extra_dirs):
        for extra_dir_name, extra_dir_path in extra_dirs.items():
            local_path = os.path.join(local_dir, extra_dir_name)
            self.runcmd(["rm", "-rf", local_path])

    def handle_imp(self, name, build_args, **kwargs):  # noqa
        docker_image = self.get_config('/DOCKER/images/%s/image' % name, name)
        docker_file = self.get_config(
            '/DOCKER/images/%s/docker_file' % name,
            "Dockerfile"
        )
        build_dir = self.get_config("/DOCKER/images/%s/build_dir" % name)
        extra_dirs = self.get_config("/DOCKER/images/%s/extra_dirs" % name, {})
        self._copy_extra_dirs(build_dir, extra_dirs)

        try:
            self.runcmd(
                [
                    "docker",
                    "build",
                    "-t",
                    docker_image,
                    "-f",
                    docker_file,
                ] + remove_trailing_dashes(build_args) +
                [
                    ".",
                ],
                cwd=build_dir
            )
        finally:
            self._remove_extra_dirs(build_dir, extra_dirs)



"""Compare configuration version to version in original project config file."""
from . import DodoCommand
from dodo_commands.framework import get_version
from dodo_commands.framework.util import bordered
from semantic_version import Version
import os
import ruamel.yaml
import sys


def _get_root_config_field(config_filename, field_name):
    with open(config_filename) as f:
        config = ruamel.yaml.round_trip_load(f.read())
    return config.get("ROOT", {}).get(field_name, "")


def _partial_sem_version(version):
    v = Version(version)
    return Version("%s.%s" % (v.major, v.minor), partial=True)


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            '--dodo',
            dest='check_dodo',
            action='store_true',
            help=(
                "Check that the installed dodo commands version satisfies the "
                "minimal dodo commands version in the dodo config "
                "(/ROOT/required_dodo_commands_version)"
            )
        )

        parser.add_argument(
            '--config',
            dest='check_config',
            action='store_true',
            help=(
                "Check that the version field in the local dodo commands config "
                "(/ROOT/version) is up-to-date with the version in the shared "
                "dodo commands config."
            )
        )

    def _config_filename(self):
        return os.path.join(
            self.get_config("/ROOT/project_dir", ""),
            "dodo_commands",
            "res",
            "config.yaml"
        )

    def check_dodo_commands_version(self):  # noqa
        required_version = _get_root_config_field(
            self._config_filename(), 'required_dodo_commands_version'
        )
        if required_version:
            actual_version = get_version()
            if Version(actual_version) < Version(required_version):
                sys.stdout.write(bordered(
                    'The dodo_commands package needs to be upgraded (%s < %s). Tip: use "dodo upgrade"'
                    % (
                        actual_version,
                        required_version,
                    ),
                ))
                sys.stdout.write('\n')

    def check_config_version(self):  # noqa
        project_dir = self.get_config("/ROOT/project_dir", "")
        original_file = os.path.join(
            project_dir, "dodo_commands", "default_project", "config.yaml"
        )
        if not os.path.exists(original_file):
            return

        original_version = _get_root_config_field(original_file, 'version')
        if not original_version:
            sys.stderr.write(
                "No version found in original file %s\n" % original_file
            )
            return

        copied_file = os.path.join(project_dir, "dodo_commands", "res", "config.yaml")
        copied_version = _get_root_config_field(copied_file, 'version')
        if not copied_version:
            sys.stderr.write(
                "No version found in user managed file %s\n" % copied_file
            )
            return

        if (
            _partial_sem_version(copied_version) <
            _partial_sem_version(original_version)
        ):
            sys.stdout.write(bordered(
                'Configuration needs update (%s < %s). Tip: use "dodo diff ."\n'
                % (
                    copied_version,
                    original_version,
                )
            ))
            sys.stdout.write('\n')

    def handle_imp(self, check_dodo, check_config, **kwargs):  # noqa
        if check_dodo:
            self.check_dodo_commands_version()
        if check_config:
            self.check_config_version()
